@@ -21,9 +21,16 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <ctype.h>
+#ifdef __QNX__
+#include <X11/IntrinsicP.h> /* for XtResizeWidget() */
+#include <unix.h> /* for gethostname() */
+#endif
 
 #include "pop.h"
 
+/* Externals **************************************************************** */
+extern char *safemalloc(int length);
+
 /* XPM struct and icons ***************************************************** */
 typedef struct _XpmIcon {
     Pixmap pixmap;
@@ -32,8 +39,9 @@ typedef struct _XpmIcon {
     struct XpmIcon *next;
 } XpmIcon;
 
-XpmIcon *NoMail, *OldMail, *NewMail, *Current;
+XpmIcon *Frame = NULL, *NoMail = NULL, *OldMail = NULL, *NewMail = NULL, *Current;
 
+#include "pixmaps/frame.xpm"
 #include "pixmaps/newmail.xpm"
 #include "pixmaps/oldmail.xpm"
 #include "pixmaps/nomail.xpm"
@@ -45,7 +53,6 @@ XpmIcon *NoMail, *OldMail, *NewMail, *Current;
 int CheckMailBox(char **Mbox, int NumOfMbox);
 int IsNewMail(char *Mbox, int index);
 int IsMail(char *Mbox, int index);
-int NoOfNewMail(char *Mbox, int index);
 int CheckTimeStamp(char **Mbox, int NumOfMbox);
 int GetGStatus(void);
 void CheckMailCallback(XtPointer XtP, XtIntervalId * XtI);
@@ -54,15 +61,18 @@ void CreateWindow(int argc, char *argv[]);
 void MakeSettings(int argc, char *argv[]);
 void ParseCfgFile(FILE * File);
 void StartLoop(void);
-void GetXPM(int Shaped);
+XpmIcon *GetXPM(char **data);
+void FreeXPM(XpmIcon *Icon);
 void LoadXPMFile(XpmIcon * Icon, char *XpmFile);
 void XPMError(int Code);
 void RedrawWindow(void);
 void ButtonHandler(Widget W, XtPointer P, XEvent * E);
 void ExposeHandler(Widget W, XtPointer P, XEvent * E);
+void StructureNotifyHandler(Widget W, XtPointer P, XEvent * E);
 void AnimCallback(XtPointer XtP, XtIntervalId * XtI);
 Pixel GetColor(char *name);
 void AddXpmToAnim(XpmIcon * Icon, XpmIcon * List, int ListNo);
+void Quit(Widget w, XEvent *event, String *params, Cardinal *num_params);
 
 int IsPopBox(char *string);
 int TotalNewMail(void);
@@ -72,12 +82,18 @@ int TotalMail(void);
 /* Global stuff ************************************************************* */
 #define FILENAME		".asmailrc"
 #define DEFINTERVAL		5	/* Default interval 5 sec */
-#define DEFANIMATIONSPEED	10	/* Default animationspeed 10 */
+#define DEFANIMATIONSPEED	0	/* Default animationspeed is none */
 #define LEFTBUTTON		1	/* Left mousebutton id */
 #define RIGHTBUTTON		3	/* Right mousebutton id */
 #define MAXNUMOFMAILBOXES	256	/* Maximim number of mailboxes */
+#define XPMCLOSENESS 32768
 Display *Disp = 0;
 Window Root;
+XtActionsRec asmail_actions[] =
+{
+    { "Quit", Quit }
+};
+Atom wm_delete_win;
 char Execute[256] = "";
 char NewMailExecute[256] = "";
 char ExecuteOnUpdate[256] = "";
@@ -97,10 +113,12 @@ int TimeStampMode = 0;
 int LockShape = 0;
 int *IStatus = 0;
 int *GStatus = 0;
+int IsMapped;
 int NumOfMsgX = 10;
 int NumOfMsgY = 10;
 int NumOfMsgMode = 0;
 XTextItem NumOfMsg;
+Pixel NumOfMsgColor = -1; /* defaults to white? */
 GC WinGC;
 Widget MainWin;
 XtAppContext AsmailApp;
@@ -110,22 +128,24 @@ int AnimationSpeed[3] =
  DEFANIMATIONSPEED};
 char HelpMessage[] =
 {
-    "asmail - Version 0.52, by Per Liden (per@oden.rsn.hk-r.se)\n"
+    "asmail - Version 0.52.1, by Per Liden (per@oden.rsn.hk-r.se)\n"
     "usage:  asmail [file] [options ...] \n"
     "file:\n"
     "        configuration file to use (default ~/.asmailrc)\n"
     "options:\n"
-    "        -geometry <geom>     geometry of asmail\n"
+    "        -geometry <geom>     position of asmail\n"
     "        -display  <disp>     specifies the display to use\n"
     "        -title    <string>   title string\n"
- "        -name     <string>   client instance, icon and title strings\n"
-    "\n"};
+    "        -name     <string>   client instance, icon and title strings\n"
+    "\n"
+};
 
 /*****************************************************************************/
-void main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
     CreateWindow(argc, argv);
     StartLoop();
+    return 0; /* This will never happen */
 }
 
 /*****************************************************************************/
@@ -141,6 +161,7 @@ void MakeSettings(int argc, char *argv[])
 {
     char CfgFile[256];
     FILE *File = 0;
+	int i;
 
     /* Get config file */
     if (argc == 1) {
@@ -170,18 +191,10 @@ void MakeSettings(int argc, char *argv[])
 	fprintf(stderr, "asmail: invalid update frequency specified, using default\n");
 	Interval = DEFINTERVAL;	/* Use default */
     }
-    /* Check animationspeed */
-    if (!AnimationSpeed[0] || !AnimationSpeed[1] || !AnimationSpeed[2]) {
-	fprintf(stderr, "asmail: invalid animationspeed specified, using default\n");
-	AnimationSpeed[0] = DEFANIMATIONSPEED;	/* Use default */
-	AnimationSpeed[1] = DEFANIMATIONSPEED;	/* Use default */
-	AnimationSpeed[2] = DEFANIMATIONSPEED;	/* Use default */
-    }
-    AnimationSpeed[0] *= 10;
-    AnimationSpeed[1] *= 10;
-    AnimationSpeed[2] *= 10;
-
-    /* If no MaiiFile option search for $MAIL */
+    /* Check animation speed */
+	for (i = 0; i < 3; i++)
+    	if (AnimationSpeed[i]) AnimationSpeed[i] *= 10;
+    /* If no MailFile option search for $MAIL */
     if (NumOfMailBoxes == 0) {
 	   if (getenv("MAIL") != NULL) {
 	      NumOfMailBoxes = 1;
@@ -201,6 +214,30 @@ void MakeSettings(int argc, char *argv[])
 	fprintf(stderr, "asmail: invalid NumOfMsgMode specified\n");
 	exit(1);
     }
+	/* Load or discard default XPM files and unset animation as required */
+	if (Shape && Frame)
+		FreeXPM(Frame);
+	else if (!Shape && !Frame)
+		Frame = GetXPM(frame);
+	if (!NoMail)
+	{
+		NoMail = GetXPM((Shape) ? nomail_s : nomail);
+		NoMail->next = (struct XpmIcon *)NoMail;
+		AnimationSpeed[0] = 0;
+		Current = NoMail;
+	}
+	if (!OldMail)
+	{
+		OldMail = GetXPM((Shape) ? oldmail_s : oldmail);
+		OldMail->next = (struct XpmIcon *)OldMail;
+		AnimationSpeed[1] = 0;
+	}
+	if (!NewMail)
+	{
+		NewMail = GetXPM((Shape) ? newmail_s : newmail);
+		NewMail->next = (struct XpmIcon *)NewMail;
+		AnimationSpeed[2] = 0;
+	}
 }
 
 /****************************************************************************/
@@ -256,23 +293,31 @@ void shorten(char *line)
     line[++i] = '\0';
 }
 /****************************************************************************/
+int isComment(FILE *file, char *buffer)
+{
+	if (buffer[0] == '#' || buffer[0] == '\0')
+	{
+		/* Goto next line */
+		fgets(buffer, 255, file);
+		return 1;
+	}
+	return 0;
+}
+/****************************************************************************/
 void ParseCfgFile(FILE * File)
 {
     char Buffer[256];
     int i;
     char c;
     XpmIcon *Icon;
-    int PopMail = 0;
 
     while (!feof(File)) {
 	/* Read string */
 	fscanf(File, "%s", Buffer);
 
 	/* Comment */
-	if (Buffer[0] == '#') {
-	    /* Got to next line */
-	    fgets(Buffer, 255, File);
-	}
+	if (isComment(File, Buffer))
+		;
 	/* NoBeep */
 	else if (!strcasecmp(Buffer, "NoBeep")) {
 	    Beep = 0;
@@ -280,7 +325,6 @@ void ParseCfgFile(FILE * File)
 	/* NoShape */
 	else if (!strcasecmp(Buffer, "NoShape")) {
 	    Shape = 0;
-	    GetXPM(0);
 	}
 	/* LockShape */
 	else if (!strcasecmp(Buffer, "LockShape")) {
@@ -291,6 +335,7 @@ void ParseCfgFile(FILE * File)
 	    i = 0;
 	    while (1) {
     	   fgets(Buffer, 256, File);
+           if (Buffer[0] == '#' || Buffer[0] == '\0') continue;
            shorten(Buffer);
            /* End */
            if (!strcasecmp(Buffer, "End")) {
@@ -310,14 +355,14 @@ void ParseCfgFile(FILE * File)
            else {
                /* Check maximin number of mailboxes */
                if (i > MAXNUMOFMAILBOXES) {
-               fprintf(stderr, "asmail: too many mailboxes specified");
-               exit(1);
+                   fprintf(stderr, "asmail: too many mailboxes specified");
+                   exit(1);
+               }
+               /* Add mailbox */
+               MailBoxes[i] = (char *) calloc(strlen(Buffer) + 1, sizeof(char));
+               strcpy(MailBoxes[i], Buffer);
+               i++;
            }
-	    /* Add mailbox */
-           MailBoxes[i] = (char *) calloc(strlen(Buffer) + 1, sizeof(char));
-		   strcpy(MailBoxes[i], Buffer);
-		   i++;
-		}
 	    }
 	}
 	/* Update <sec> */
@@ -353,6 +398,12 @@ void ParseCfgFile(FILE * File)
 	    fscanf(File, "%s ", Buffer);
 	    NumOfMsg.font = XLoadFont(Disp, Buffer);
 	}
+    /* NumOfMsgColor <string> */
+    else if (!strcasecmp(Buffer, "NumOfMsgColor")) {
+        fscanf(File, "%s ", Buffer);
+        if (!(NumOfMsgColor = GetColor(Buffer)))
+            NumOfMsgColor = GetColor("white");
+	}
 	/* NewMailExecute <program> */
 	else if (!strcasecmp(Buffer, "NewMailExecute")) {
 	    fgetc(File);
@@ -367,17 +418,27 @@ void ParseCfgFile(FILE * File)
 	else if (!strcasecmp(Buffer, "TimeStampMode")) {
 	    TimeStampMode = 1;
 	}
-	/* AnimationSpeed <nomail> <oldmail <newmail> */
+	/* AnimationSpeed <nomail> <oldmail> <newmail> */
 	else if (!strcasecmp(Buffer, "AnimationSpeed")) {
 	    fscanf(File, "%d ", &AnimationSpeed[0]);
 	    fscanf(File, "%d ", &AnimationSpeed[1]);
 	    fscanf(File, "%d ", &AnimationSpeed[2]);
 	}
+	/* Frame <XPM file> */
+	else if (!strcasecmp(Buffer, "Frame")) {
+		free(Frame);
+		fscanf(File, "%s ", Buffer);
+		if (!Shape) {
+			Frame = (XpmIcon *) calloc(1, sizeof(XpmIcon));
+			LoadXPMFile(Frame, Buffer);
+		}
+	}
 	/* NoMail <...> */
 	else if (!strcasecmp(Buffer, "NoMail")) {
 	    free(NoMail);
 	    while (1) {
 		fscanf(File, "%s ", Buffer);
+		if (isComment(File, Buffer)) continue;
 		if (!strcasecmp(Buffer, "End"))
 		    break;
 		Icon = (XpmIcon *) calloc(1, sizeof(XpmIcon));
@@ -391,6 +452,7 @@ void ParseCfgFile(FILE * File)
 	    free(OldMail);
 	    while (1) {
 		fscanf(File, "%s ", Buffer);
+		if (isComment(File, Buffer)) continue;
 		if (!strcasecmp(Buffer, "End"))
 		    break;
 		Icon = (XpmIcon *) calloc(1, sizeof(XpmIcon));
@@ -403,6 +465,7 @@ void ParseCfgFile(FILE * File)
 	    free(NewMail);
 	    while (1) {
 		fscanf(File, "%s ", Buffer);
+		if (isComment(File, Buffer)) continue;
 		if (!strcasecmp(Buffer, "End"))
 		    break;
 		Icon = (XpmIcon *) calloc(1, sizeof(XpmIcon));
@@ -421,12 +484,22 @@ void ParseCfgFile(FILE * File)
 	}
     }
 }
-
+/****************************************************************************/
+void Quit(Widget w, XEvent *event, String *params, Cardinal *num_params)
+{
+    if (event->type == ClientMessage &&
+        event->xclient.data.l[0] != wm_delete_win) {
+        XBell(Disp, 0);
+        return;
+    }
+    XFreeGC(Disp, WinGC);
+    XCloseDisplay(Disp);
+    exit (0);
+}
 /****************************************************************************/
 void CreateWindow(int argc, char *argv[])
 {
     XSizeHints SizeHints;
-    XWMHints WmHints;
     XGCValues gcv;
     unsigned long gcm;
 
@@ -438,16 +511,34 @@ void CreateWindow(int argc, char *argv[])
     /* Add eventhandlers */
     XtAddEventHandler(MainWin, ExposureMask, FALSE, (XtEventHandler) ExposeHandler, NULL);
     XtAddEventHandler(MainWin, ButtonPressMask, FALSE, (XtEventHandler) ButtonHandler, NULL);
+    XtAddEventHandler(MainWin, StructureNotifyMask, FALSE,
+                   (XtEventHandler) StructureNotifyHandler, NULL);
 
-    /* Get XPM data */
-    GetXPM(1);
+    /* Add actions */
+    XtAppAddActions(AsmailApp, asmail_actions, XtNumber(asmail_actions));
+    XtOverrideTranslations(MainWin,
+                   XtParseTranslationTable("<Message>WM_PROTOCOLS: Quit()"));
 
     /* Make settings */
     MakeSettings(argc, argv);
 
     /* Set window attributes */
-    XtResizeWidget(MainWin, Current->attributes.width, Current->attributes.height, 1);
+    if (Frame)
+    {
+        SizeHints.width = Frame->attributes.width;
+        SizeHints.height = Frame->attributes.height;
+    }
+    else
+    {
+        SizeHints.width = Current->attributes.width;
+        SizeHints.height = Current->attributes.height;
+    }
+    XtResizeWidget(MainWin, SizeHints.width, SizeHints.height, 1);
     XtRealizeWidget(MainWin);
+    SizeHints.flags = USPosition | PSize | PMinSize | PMaxSize;
+    SizeHints.min_width = SizeHints.max_width = SizeHints.width;
+    SizeHints.min_height = SizeHints.max_height = SizeHints.height;
+    XSetWMNormalHints(Disp, XtWindow(MainWin), &SizeHints);
 
     /* Create GC */
     gcm = GCForeground | GCBackground | GCGraphicsExposures;
@@ -456,9 +547,12 @@ void CreateWindow(int argc, char *argv[])
     gcv.graphics_exposures = False;
     WinGC = XCreateGC(Disp, Root, gcm, &gcv);
 
+    /* Add protocol messages */
+    wm_delete_win = XInternAtom(Disp, "WM_DELETE_WINDOW", False);
+    (void) XSetWMProtocols(Disp, XtWindow(MainWin), &wm_delete_win, 1);
+
     /* Map mainwindow */
     XtMapWidget(MainWin);
-    RedrawWindow();
 }
 /****************************************************************************/ 
 int TotalNewMail(void)
@@ -480,10 +574,51 @@ int TotalMail(void)
     
     return c;
 }
+/****************************************************************************/ 
+void WMIconName(void)
+{
+	static Window wnd = None;
+	static int lastall = 0, lastnew = 0;
+	int all = TotalMail(), new = TotalNewMail();
+	char *w_name, i_name[50];
+
+#if 0
+	printf("%d/%d\n", all, new);
+#endif
+
+	if (lastall == all && lastnew == new) return;
+	lastall = all;
+	lastnew = new;
+
+	if (wnd == None) wnd = XtWindow(MainWin);
+
+	XFetchName(Disp, wnd, &w_name);
+
+	if (all > 0)
+	{
+		if (new > 0)
+			sprintf(i_name, "%s (%d new in %d)", w_name, new, all);
+		else
+			sprintf(i_name, "%s (%d msg%s)", w_name, all,
+					(all != 1) ? "s" : "");
+
+		XSetIconName(Disp, wnd, i_name);
+	}
+	else
+		XSetIconName(Disp, wnd, w_name);
+
+	XFree(w_name);
+}
 /***************************************************************************/
 void RedrawWindow(void)
 {
+    XGCValues gcv;
     int GS = GetGStatus();
+    static int LockFrame = 0, x = 0, y = 0;
+/*
+    static int lastall = 0, lastnew = 0;
+*/
+    int all, new;
 
     /* Select icon */
     switch (GS) {
@@ -503,27 +638,52 @@ void RedrawWindow(void)
 	if (LockShape)
 	    Shape = 0;
     }
+    else if (Frame && !LockFrame)
+    {
+        XCopyArea(Disp, Frame->pixmap, XtWindow(MainWin), WinGC, 0, 0,
+                Frame->attributes.width, Frame->attributes.height, 0, 0);
+        x = (Frame->attributes.width - Current->attributes.width) / 2;
+        y = (Frame->attributes.height - Current->attributes.height) / 2;
+        if (LockShape) LockFrame = 1;
+    }
+
     /* Draw icon */
     XCopyArea(Disp, Current->pixmap, XtWindow(MainWin), WinGC, 0, 0,
-	    Current->attributes.width, Current->attributes.height, 0, 0);
+	    Current->attributes.width, Current->attributes.height, x, y);
 
-    /* Draw number of messages */
-    if (GS && NumOfMsgMode) {
-	/* Create string */
-	switch (NumOfMsgMode) {
-	case 1:
-	    sprintf(NumOfMsg.chars, "%d/%d", TotalNewMail(), TotalMail());
-	    break;
-	case 2:
-	    sprintf(NumOfMsg.chars, "%d", TotalNewMail());
-	    break;
-	case 3:
-	    sprintf(NumOfMsg.chars, "%d", TotalMail());
-	    break;
+	/* Draw number of messages */
+	if (GS && NumOfMsgMode) {
+		all = TotalMail();
+		new = TotalNewMail();
+/*
+		if (lastall != all || lastnew != new) {
+			lastall = all;
+			lastnew = new;
+*/
+			/* Create string */
+			switch (NumOfMsgMode) {
+				case 1:
+					sprintf(NumOfMsg.chars, "%d/%d", new, all);
+					break;
+				case 2:
+					sprintf(NumOfMsg.chars, "%d", new);
+					break;
+				case 3:
+					sprintf(NumOfMsg.chars, "%d", all);
+					break;
+			}
+			gcv.foreground = NumOfMsgColor;
+			XChangeGC(Disp, WinGC, GCForeground, &gcv);
+			NumOfMsg.nchars = strlen(NumOfMsg.chars);
+			XDrawText(Disp, XtWindow(MainWin), WinGC, x + NumOfMsgX, y + NumOfMsgY,
+					&NumOfMsg, 1);
+/*
+		}
+*/
 	}
-	NumOfMsg.nchars = strlen(NumOfMsg.chars);
-	XDrawText(Disp, XtWindow(MainWin), WinGC, NumOfMsgX, NumOfMsgY, &NumOfMsg, 1);
-    }
+
+	/* Update WM icon name */
+	WMIconName();
 }
 
 /****************************************************************************/
@@ -546,7 +706,7 @@ void ButtonHandler(Widget W, XtPointer P, XEvent * E)
 		} else {
 		    GStatus[i] = 1;
 		    IStatus[i] = 3;
-		    NewMailCounter[i] = NoOfNewMail(MailBoxes[i], i);
+		    NewMailCounter[i] = IsNewMail(MailBoxes[i], i);
 		}
 		Redraw++;
 	    }
@@ -559,22 +719,56 @@ void ButtonHandler(Widget W, XtPointer P, XEvent * E)
 /****************************************************************************/
 void ExposeHandler(Widget W, XtPointer P, XEvent * E)
 {
+    if (Frame)
+        XCopyArea(Disp, Frame->pixmap, XtWindow(MainWin), WinGC, 0, 0,
+                Frame->attributes.width, Frame->attributes.height, 0, 0);
     RedrawWindow();
-} /****************************************************************************/ 
+}
+
+/****************************************************************************/
+void StructureNotifyHandler(Widget W, XtPointer P, XEvent * E)
+{
+	if (E->type == MapNotify)
+		IsMapped = 1;
+	else if (E->type == UnmapNotify)
+		IsMapped = 0;
+}
+
+/****************************************************************************/ 
+void SetAnimTimeout(void)
+{
+	int i, j = 0;
+
+	if (IsMapped)
+	{
+		/* Are there any animated statuses? */
+		for (i = 0; i < 3; i++)
+			if ((j = AnimationSpeed[i])) break;
+		/* Is this status animated? */
+		i = AnimationSpeed[GetGStatus()];
+	}
+	else
+		i = 1000; /* 1 second */
+
+	/* If unmapped, use a longish value. Otherwise, if there    */
+	/* are animations, use this status's value before any other */
+	if (i || j)
+	   	XtAppAddTimeOut(AsmailApp, (unsigned long) ((i) ? i : j),
+			    (XtTimerCallbackProc) AnimCallback, NULL);
+}
+/****************************************************************************/ 
 void StartLoop(void)
 {
     /* Check mailbox at statup */
-    RedrawWindow();
-    /*if (CheckMailBox(MailBoxes, NumOfMailBoxes))
-	   RedrawWindow();*/
+    if (CheckMailBox(MailBoxes, NumOfMailBoxes))
+	   RedrawWindow();
 
     /* Set timeout */
-    XtAppAddTimeOut(AsmailApp, Interval * 1000,
+    XtAppAddTimeOut(AsmailApp, (unsigned long) (Interval * 1000),
 		    (XtTimerCallbackProc) CheckMailCallback, NULL);
 
-    /* Set animation timeout's */
-    XtAppAddTimeOut(AsmailApp, AnimationSpeed[GetGStatus()],
-		    (XtTimerCallbackProc) AnimCallback, NULL);
+    /* Set animation timeout */
+    SetAnimTimeout();
 
     /* Start looping */
     XFlush(Disp);
@@ -583,18 +777,20 @@ void StartLoop(void)
 /****************************************************************************/
 void AnimCallback(XtPointer XtP, XtIntervalId * XtI)
 {
+	int aGS = AnimationSpeed[GetGStatus()];
+
     /* Change to next image */
     NoMail = (XpmIcon *) NoMail->next;
     OldMail = (XpmIcon *) OldMail->next;
     NewMail = (XpmIcon *) NewMail->next;
 
     /* Redraw */
-    RedrawWindow();
+	if (IsMapped && aGS) RedrawWindow();
 
     /* Set animation timeout */
-    XtAppAddTimeOut(AsmailApp, AnimationSpeed[GetGStatus()],
-		    (XtTimerCallbackProc) AnimCallback, NULL);
-} /****************************************************************************/
+    SetAnimTimeout();
+}
+/****************************************************************************/
 void CheckMailCallback(XtPointer XtP, XtIntervalId * XtI)
 {
     /* Check if ExecuteOnUpdate */
@@ -608,7 +804,7 @@ void CheckMailCallback(XtPointer XtP, XtIntervalId * XtI)
 	if (CheckMailBox(MailBoxes, NumOfMailBoxes))
 	    RedrawWindow();
     }				/* Set timeout */
-    XtAppAddTimeOut(AsmailApp, Interval * 1000,
+    XtAppAddTimeOut(AsmailApp, (unsigned long) (Interval * 1000),
 		    (XtTimerCallbackProc) CheckMailCallback, NULL);
 }
 /************************************************************/
@@ -658,7 +854,7 @@ int CheckMailBox(char **Mbox, int NumOfMbox)
     static int iCheckingMailbox = 0;
     int i;
     int Prev = 0;
-    int oNewMailCounter = 0;
+    int oNewMailCounter = 0, nNewMailCounter = 0;
     int Result = 0;
     int BeepFlag = 0;
     int ExecFlag = 0;
@@ -670,8 +866,11 @@ int CheckMailBox(char **Mbox, int NumOfMbox)
         oNewMailCounter = NewMailCounter[i];
 /************************************************************/
 	    if (IsMail(Mbox[i], i)) { /* There is a kind of mail waiting */
-            if (IsNewMail(Mbox[i], i)) {
-		          if (NoOfNewMail(Mbox[i], i) > oNewMailCounter) {
+            if ((nNewMailCounter = IsNewMail(Mbox[i], i))) {
+/* Allow for shrinking new mail counts
+		          if (nNewMailCounter > oNewMailCounter) {
+*/
+		          if (nNewMailCounter != oNewMailCounter) {
                       if (IsNew) {
                           if (IStatus[i] == 2 && AlwaysNewMailExecute && NewMailExecuteFlag)
                               ExecFlag = 1;
@@ -680,7 +879,7 @@ int CheckMailBox(char **Mbox, int NumOfMbox)
                           BeepFlag = 1;
                           IsNew = 0;
                           if (!IsPopBox(Mbox[i]))
-                              NewMailCounter[i] = NoOfNewMail(Mbox[i], i);
+                              NewMailCounter[i] = nNewMailCounter;
                       }
                       IStatus[i] = 2;
                       GStatus[i] = 2;
@@ -708,8 +907,8 @@ int CheckMailBox(char **Mbox, int NumOfMbox)
 	    }
 /************************************************************/
 	    iCheckingMailbox = 0;
+	    Result |= (Prev != GStatus[i]);
 	}
-	Result |= (Prev != GStatus[i]);
   }
   /* Check for execute and beep */
   if (ExecFlag)
@@ -720,48 +919,18 @@ int CheckMailBox(char **Mbox, int NumOfMbox)
   return Result;
 }
 
-/****************************************************************************/
-int NoOfNewMail(char *Mbox, int index)
-{
-    FILE *MailFile = 0;
-    char Buffer[30] = "";
-    int NewMailn = 0;
-    
-    if (IsPopBox(Mbox))
-        return NewMailCounter[index];
-
-    MailFile = fopen(Mbox, "r");
-    MailCounter[index] = 0;
-
-    if (MailFile == NULL)
-	   return 0;
-
-    while (!feof(MailFile)) {
-	   fgets(Buffer, 29, MailFile);
-       if (!strncmp(Buffer, "From ", 5)) {
-           MailCounter[index]++;
-           NewMailn++;
-       } else if (!strncmp(Buffer, "Status: RO", 10))
-           NewMailn--;
-    }
-    fclose(MailFile);
-
-    return NewMailn;
-}
-
 /****************************************************************************/
 int IsNewMail(char *Mbox, int index)
 {
     FILE *MailFile = 0;
     char Buffer[30] = "";
-    int WasStatus = 1;
     int NewMailn = 0;
     
     if (IsPopBox(Mbox))
         return NewMailCounter[index];
     
-    MailCounter[index] = 0;
     MailFile = fopen(Mbox, "r");
+    MailCounter[index] = 0;
 
     if (MailFile == NULL)
 	   return 0;
@@ -770,20 +939,22 @@ int IsNewMail(char *Mbox, int index)
 	   fgets(Buffer, 29, MailFile);
        if (!strncmp(Buffer, "From ", 5)) {
            MailCounter[index]++;
-           if (!WasStatus)
                NewMailn++;
-           WasStatus = 0;
+/* Changes meaning of "new mail" to "unread mail"
        } else if (!strncmp(Buffer, "Status: RO", 10))
-           WasStatus = 1;
+*/
+       } else if (!strncmp(Buffer, "Status: R", 9))
+           NewMailn--;
     }
     fclose(MailFile);
 
-    return (NewMailn || !WasStatus);
+    return (NewMailn);
 }
 /****************************************************************************/
 int IsPopBox(char *string)
 {
-    if (string[0] == ':' && string[1] == ':')
+    if ((string[0] == ':' && string[1] == ':') ||
+        (string[0] == '>' && string[1] == '>'))
         return 1;
     else
         return 0;
@@ -798,7 +969,7 @@ int IsMail(char *Mbox, int index)
 
     if (IsPopBox(Mbox)) {
         Exec = (char *)malloc(strlen(Mbox));
-        strncpy(Exec, Mbox+2, strlen(Mbox)-1);
+        strncpy(Exec, Mbox, strlen(Mbox)+1);
         
         if (Exec == NULL)
             return 0;
@@ -840,94 +1011,73 @@ int GetGStatus(void)
 	    j = GStatus[i];
     return j;
 }
+
 /****************************************************************************/
-void GetXPM(int Shaped)
+XpmIcon *GetXPM(char **data)
 {
-    /* Create default icons */
-    if (Shaped) {
-	/* Shaped */
-	NoMail = (XpmIcon *) calloc(1, sizeof(XpmIcon));
-	OldMail = (XpmIcon *) calloc(1, sizeof(XpmIcon));
-	NewMail = (XpmIcon *) calloc(1, sizeof(XpmIcon));
+	XpmIcon *icon = (XpmIcon *) calloc(1, sizeof(XpmIcon));
+
+	icon->attributes.valuemask |= XpmCloseness;
+	icon->attributes.closeness = XPMCLOSENESS;
 
 	XPMError(XpmCreatePixmapFromData(Disp, Root,
-					 nomail_s,
-					 &NoMail->pixmap,
-					 &NoMail->mask,
-					 &NoMail->attributes));
-	XPMError(XpmCreatePixmapFromData(Disp, Root,
-					 newmail_s,
-					 &NewMail->pixmap,
-					 &NewMail->mask,
-					 &NewMail->attributes));
-	XPMError(XpmCreatePixmapFromData(Disp, Root,
-					 oldmail_s,
-					 &OldMail->pixmap,
-					 &OldMail->mask,
-					 &OldMail->attributes));
+					 data,
+					 &icon->pixmap,
+					 &icon->mask,
+					 &icon->attributes));
 
-	NoMail->next = (struct XpmIcon *) NoMail;
-	OldMail->next = (struct XpmIcon *) OldMail;
-	NewMail->next = (struct XpmIcon *) NewMail;
+	icon->next = (struct XpmIcon *) icon;
 
-	/* Set startup-status to Nomail */
-	Current = NoMail;
-    } else {
-	/* Normal */
-	XPMError(XpmCreatePixmapFromData(Disp, Root,
-					 nomail,
-					 &NoMail->pixmap,
-					 &NoMail->mask,
-					 &NoMail->attributes));
-	XPMError(XpmCreatePixmapFromData(Disp, Root,
-					 newmail,
-					 &NewMail->pixmap,
-					 &NewMail->mask,
-					 &NewMail->attributes));
-	XPMError(XpmCreatePixmapFromData(Disp, Root,
-					 oldmail,
-					 &OldMail->pixmap,
-					 &OldMail->mask,
-					 &OldMail->attributes));
-    }
+	return icon;
 }
-
 /****************************************************************************/
 void LoadXPMFile(XpmIcon * Icon, char *XpmFile)
 {
     char *Home, *realfilename;
 
-/* home dir ? */
+    /* home dir ? */
     if (strncmp(XpmFile, "~/", 2) == 0) {
-/* get home */
+	/* get home */
 	Home = getenv("HOME");
 	if (Home == NULL)
 	    Home = "./";
-/* alloc it */
+	/* alloc it */
 	realfilename = (char *) safemalloc(strlen(Home) + strlen(&XpmFile[2]) + 3);
-/* put it */
+	/* put it */
 	strcpy(realfilename, Home);
 	strcat(realfilename, "/");
 	strcat(realfilename, &XpmFile[2]);
     } else
 	realfilename = XpmFile;
 
+	Icon->attributes.valuemask |= XpmCloseness;
+	Icon->attributes.closeness = XPMCLOSENESS;
 
-    if (XpmReadFileToPixmap(Disp, Root,
+    XPMError(XpmReadFileToPixmap(Disp, Root,
 			    realfilename,
 			    &Icon->pixmap,
 			    &Icon->mask,
-			    &Icon->attributes) != XpmSuccess) {
-	fprintf(stderr, "asmail: could not load '%s'\n", realfilename);
-	exit(1);
-    }
+			    &Icon->attributes));
+
     /* Check if mask is ok */
+/* Test only if window is shaped
     if (Icon->mask == 0) {
+*/
+    if (Shape && Icon->mask == 0) {
 	fprintf(stderr, "asmail: '%s' does not have a transparent color\n", realfilename);
 	exit(1);
     }
 }
 /****************************************************************************/
+void FreeXPM(XpmIcon *Icon)
+{
+	XpmFreeAttributes(&Icon->attributes);
+	XpmFree((char *)Icon->pixmap);
+	XpmFree((char *)Icon->mask);
+
+	Icon = NULL;
+}
+/****************************************************************************/
 void XPMError(int Code)
 {
     switch (Code) {
@@ -967,4 +1117,5 @@ Pixel GetColor(char *ColorName)
 	fprintf(stderr, "asmail: can't allocate %s\n", ColorName);
 
     return Color.pixel;
-}				/* END OF FILE ************************************************************* */
+}
+/* END OF FILE ************************************************************* */
